@@ -85,6 +85,9 @@ MKMAP_OPEN_INFO mkmap_types[] = {
 #ifdef HAS_CDB
     DICT_TYPE_CDB, mkmap_cdb_open,
 #endif
+#ifdef HAS_SDBM
+    DICT_TYPE_SDBM, mkmap_sdbm_open,
+#endif
 #ifdef HAS_DBM
     DICT_TYPE_DBM, mkmap_dbm_open,
 #endif
