@@ -106,8 +106,6 @@
 /*	This service must be configured with process limit of 1.
 /* .IP MAIL_SERVER_UNLIMITED
 /*	This service must be configured with process limit of 0.
-/* .IP MAIL_SERVER_PRIVILEGED
-/*	This service must be configured as privileged.
 /* .PP
 /*	The var_use_limit variable limits the number of clients that
 /*	a server can service before it commits suicide.
@@ -152,7 +150,6 @@
 #ifdef STRCASECMP_IN_STRINGS_H
 #include <strings.h>
 #endif
-#include <time.h>
 
 /* Utility library. */
 
@@ -325,54 +322,6 @@ static void trigger_server_accept_local(int unused_event, char *context)
     close(fd);
 }
 
-#ifdef MASTER_XPORT_NAME_PASS
-
-/* trigger_server_accept_pass - accept descriptor */
-
-static void trigger_server_accept_pass(int unused_event, char *context)
-{
-    char   *myname = "trigger_server_accept_pass";
-    int     listen_fd = CAST_CHAR_PTR_TO_INT(context);
-    int     time_left = 0;
-    int     fd;
-
-    if (msg_verbose)
-	msg_info("%s: trigger arrived", myname);
-
-    /*
-     * Read a message from a socket. Be prepared for accept() to fail because
-     * some other process already got the connection. The socket is
-     * non-blocking so we won't get stuck when multiple processes wake up.
-     * Don't get stuck when the client connects but sends no data. Restart
-     * the idle timer if this was a false alarm.
-     */
-    if (var_idle_limit > 0)
-	time_left = event_cancel_timer(trigger_server_timeout, (char *) 0);
-
-    if (trigger_server_pre_accept)
-	trigger_server_pre_accept(trigger_server_name, trigger_server_argv);
-    fd = PASS_ACCEPT(listen_fd);
-    if (trigger_server_lock != 0
-	&& myflock(vstream_fileno(trigger_server_lock), INTERNAL_LOCK,
-		   MYFLOCK_OP_NONE) < 0)
-	msg_fatal("select unlock: %m");
-    if (fd < 0) {
-	if (errno != EAGAIN)
-	    msg_fatal("accept connection: %m");
-	if (time_left >= 0)
-	    event_request_timer(trigger_server_timeout, (char *) 0, time_left);
-	return;
-    }
-    close_on_exec(fd, CLOSE_ON_EXEC);
-    if (read_wait(fd, 10) == 0)
-	trigger_server_wakeup(fd);
-    else if (time_left >= 0)
-	event_request_timer(trigger_server_timeout, (char *) 0, time_left);
-    close(fd);
-}
-
-#endif
-
 /* trigger_server_main - the real main program */
 
 NORETURN trigger_server_main(int argc, char **argv, TRIGGER_SERVER_FN service,...)
@@ -557,11 +506,6 @@ NORETURN trigger_server_main(int argc, char **argv, TRIGGER_SERVER_FN service,..
 		msg_fatal("service %s requires a process limit of 0",
 			  service_name);
 	    break;
-	case MAIL_SERVER_PRIVILEGED:
-	    if (user_name)
-		msg_fatal("service %s requires privileged operation",
-			  service_name);
-	    break;
 	default:
 	    msg_panic("%s: unknown argument type: %d", myname, key);
 	}
@@ -605,10 +549,6 @@ NORETURN trigger_server_main(int argc, char **argv, TRIGGER_SERVER_FN service,..
 	    trigger_server_accept = trigger_server_accept_local;
 	else if (strcasecmp(transport, MASTER_XPORT_NAME_FIFO) == 0)
 	    trigger_server_accept = trigger_server_accept_fifo;
-#ifdef MASTER_XPORT_NAME_PASS
-	else if (strcasecmp(transport, MASTER_XPORT_NAME_PASS) == 0)
-	    trigger_server_accept = trigger_server_accept_pass;
-#endif
 	else
 	    msg_fatal("unsupported transport type: %s", transport);
     }
@@ -619,10 +559,7 @@ NORETURN trigger_server_main(int argc, char **argv, TRIGGER_SERVER_FN service,..
     if ((generation = getenv(MASTER_GEN_NAME)) != 0) {
 	if (!alldig(generation))
 	    msg_fatal("bad generation: %s", generation);
-	OCTAL_TO_UNSIGNED(trigger_server_generation, generation);
-	if (msg_verbose)
-	    msg_info("process generation: %s (%o)",
-		     generation, trigger_server_generation);
+	trigger_server_generation = strtoul(generation, (char **) 0, 8);
     }
 
     /*
@@ -668,9 +605,6 @@ NORETURN trigger_server_main(int argc, char **argv, TRIGGER_SERVER_FN service,..
      * Optionally, restrict the damage that this process can do.
      */
     resolve_local_init();
-#ifdef SNAPSHOT
-    tzset();
-#endif
     chroot_uid(root_dir, user_name);
 
     /*
