@@ -6,7 +6,7 @@
 /* SYNOPSIS
 /*	\fBpostdrop\fR [\fB-rv\fR] [\fB-c \fIconfig_dir\fR]
 /* DESCRIPTION
-/*	The \fBpostdrop\fR(1) command creates a file in the \fBmaildrop\fR
+/*	The \fBpostdrop\fR command creates a file in the \fBmaildrop\fR
 /*	directory and copies its standard input to the file.
 /*
 /*	Options:
@@ -52,7 +52,7 @@
 /*	The following \fBmain.cf\fR parameters are especially relevant to
 /*	this program.
 /*	The text below provides only a parameter summary. See
-/*	\fBpostconf\fR(5) for more details including examples.
+/*	postconf(5) for more details including examples.
 /* .IP "\fBalternate_config_directories (empty)\fR"
 /*	A list of non-default Postfix configuration directories that may
 /*	be specified with "-c config_directory" on the command line, or
@@ -72,12 +72,12 @@
 /*	records, so that "smtpd" becomes, for example, "postfix/smtpd".
 /* .IP "\fBtrigger_timeout (10s)\fR"
 /*	The time limit for sending a trigger to a Postfix daemon (for
-/*	example, the \fBpickup\fR(8) or \fBqmgr\fR(8) daemon).
+/*	example, the pickup(8) or qmgr(8) daemon).
 /* .PP
 /*	Available in Postfix version 2.2 and later:
 /* .IP "\fBauthorized_submit_users (static:anyone)\fR"
-/*	List of users who are authorized to submit mail with the \fBsendmail\fR(1)
-/*	command (and with the privileged \fBpostdrop\fR(1) helper command).
+/*	List of users who are authorized to submit mail with the sendmail(1)
+/*	command (and with the privileged postdrop(1) helper command).
 /* FILES
 /*	/var/spool/postfix/maildrop, maildrop queue
 /* SEE ALSO
@@ -152,7 +152,7 @@
  /*
   * Local mail submission access list.
   */
-char   *var_submit_acl;
+static char *var_submit_acl;
 
 static CONFIG_STR_TABLE str_table[] = {
     VAR_SUBMIT_ACL, DEF_SUBMIT_ACL, &var_submit_acl, 0, 0,
@@ -165,45 +165,40 @@ static CONFIG_STR_TABLE str_table[] = {
   */
 static char *postdrop_path;
 
+/* postdrop_cleanup - callback for the runtime error handler */
+
+static void postdrop_cleanup(void)
+{
+
+    /*
+     * This is the fatal error handler. Don't try to do anything fancy.
+     */
+    if (postdrop_path) {
+	if (remove(postdrop_path))
+	    msg_warn("uid=%ld: remove %s: %m", (long) getuid(), postdrop_path);
+	else if (msg_verbose)
+	    msg_info("remove %s", postdrop_path);
+	postdrop_path = 0;
+    }
+}
+
 /* postdrop_sig - catch signal and clean up */
 
 static void postdrop_sig(int sig)
 {
 
     /*
-     * This is the fatal error handler. Don't try to do anything fancy.
-     * 
-     * msg_vstream does not allocate memory, but msg_syslog may indirectly in
-     * syslog(), so it should not be called from a user-triggered signal
-     * handler.
-     * 
-     * Assume atomic signal() updates, even when emulated with sigaction(). We
-     * use the in-kernel SIGINT handler address as an atomic variable to
-     * prevent nested postdrop_sig() calls. For this reason, main() must
-     * configure postdrop_sig() as SIGINT handler before other signal
-     * handlers are allowed to invoke postdrop_sig().
+     * Assume atomic signal() updates, even when emulated with sigaction().
      */
-    if (signal(SIGINT, SIG_IGN) != SIG_IGN) {
-	(void) signal(SIGQUIT, SIG_IGN);
-	(void) signal(SIGTERM, SIG_IGN);
-	(void) signal(SIGHUP, SIG_IGN);
-	if (postdrop_path) {
-	    (void) remove(postdrop_path);
-	    postdrop_path = 0;
-	}
-	/* Future proofing. If you need exit() here then you broke Postfix. */
-	if (sig)
-	    _exit(sig);
+    if (signal(SIGHUP, SIG_IGN) != SIG_IGN
+	&& signal(SIGINT, SIG_IGN) != SIG_IGN
+	&& signal(SIGQUIT, SIG_IGN) != SIG_IGN
+	&& signal(SIGTERM, SIG_IGN) != SIG_IGN) {
+	postdrop_cleanup();
+	exit(sig);
     }
 }
 
-/* postdrop_cleanup - callback for the runtime error handler */
-
-static void postdrop_cleanup(void)
-{
-    postdrop_sig(0);
-}
-
 /* main - the main program */
 
 int     main(int argc, char **argv)
@@ -225,7 +220,6 @@ int     main(int argc, char **argv)
     char   *attr_name;
     char   *attr_value;
     const char *errstr;
-    char   *junk;
 
     /*
      * Be consistent with file permissions.
@@ -314,20 +308,15 @@ int     main(int argc, char **argv)
     /*
      * Set up signal handlers and a runtime error handler so that we can
      * clean up incomplete output.
-     * 
-     * postdrop_sig() uses the in-kernel SIGINT handler address as an atomic
-     * variable to prevent nested postdrop_sig() calls. For this reason, the
-     * SIGINT handler must be configured before other signal handlers are
-     * allowed to invoke postdrop_sig().
      */
     signal(SIGPIPE, SIG_IGN);
     signal(SIGXFSZ, SIG_IGN);
 
+    if (signal(SIGHUP, SIG_IGN) == SIG_DFL)
+	signal(SIGHUP, postdrop_sig);
     signal(SIGINT, postdrop_sig);
     signal(SIGQUIT, postdrop_sig);
     signal(SIGTERM, postdrop_sig);
-    if (signal(SIGHUP, SIG_IGN) == SIG_DFL)
-	signal(SIGHUP, postdrop_sig);
     msg_cleanup(postdrop_cleanup);
 
     /* End of initializations. */
@@ -406,8 +395,6 @@ int     main(int argc, char **argv)
 	    while ((rec_type = rec_get(VSTREAM_IN, buf, var_line_limit)) > 0
 		   && rec_type != REC_TYPE_END)
 		 /* void */ ;
-	    if (rec_type <= 0)
-		msg_fatal("uid=%ld: malformed input", (long) uid);
 	    break;
 	}
 	if (rec_type == REC_TYPE_END)
@@ -428,9 +415,8 @@ int     main(int argc, char **argv)
      * will not be deleted after we have taken responsibility for delivery.
      */
     if (postdrop_path) {
-	junk = postdrop_path;
+	myfree(postdrop_path);
 	postdrop_path = 0;
-	myfree(junk);
     }
 
     /*
