@@ -10,7 +10,7 @@
 /* .br
 /*	\fBpostqueue\fR [\fB-c \fIconfig_dir\fR] \fB-s \fIsite\fR
 /* DESCRIPTION
-/*	The \fBpostqueue\fR(1) command implements the Postfix user interface
+/*	The \fBpostqueue\fR program implements the Postfix user interface
 /*	for queue management. It implements operations that are
 /*	traditionally available via the \fBsendmail\fR(1) command.
 /*	See the \fBpostsuper\fR(1) command for queue operations
@@ -25,7 +25,7 @@
 /* .IP \fB-f\fR
 /*	Flush the queue: attempt to deliver all queued mail.
 /*
-/*	This option implements the traditional "\fBsendmail -q\fR" command,
+/*	This option implements the traditional \fBsendmail -q\fR command,
 /*	by contacting the Postfix \fBqmgr\fR(8) daemon.
 /*
 /*	Warning: flushing undeliverable mail frequently will result in
@@ -51,13 +51,11 @@
 /* .RE
 /* .IP "\fB-s \fIsite\fR"
 /*	Schedule immediate delivery of all mail that is queued for the named
-/*	\fIsite\fR. A numerical site must be specified as a valid RFC 2821
-/*	address literal enclosed in [], just like in email addresses.
-/*	The site must be eligible for the "fast flush" service.
+/*	\fIsite\fR. The site must be eligible for the "fast flush" service.
 /*	See \fBflush\fR(8) for more information about the "fast flush"
 /*	service.
 /*
-/*	This option implements the traditional "\fBsendmail -qR\fIsite\fR"
+/*	This option implements the traditional \fBsendmail -qR\fIsite\fR
 /*	command, by contacting the Postfix \fBflush\fR(8) daemon.
 /* .IP \fB-v\fR
 /*	Enable verbose logging for debugging purposes. Multiple \fB-v\fR
@@ -90,7 +88,7 @@
 /*	The following \fBmain.cf\fR parameters are especially relevant to
 /*	this program.
 /*	The text below provides only a parameter summary. See
-/*	\fBpostconf\fR(5) for more details including examples.
+/*	postconf(5) for more details including examples.
 /* .IP "\fBalternate_config_directories (empty)\fR"
 /*	A list of non-default Postfix configuration directories that may
 /*	be specified with "-c config_directory" on the command line, or
@@ -115,7 +113,7 @@
 /*	records, so that "smtpd" becomes, for example, "postfix/smtpd".
 /* .IP "\fBtrigger_timeout (10s)\fR"
 /*	The time limit for sending a trigger to a Postfix daemon (for
-/*	example, the \fBpickup\fR(8) or \fBqmgr\fR(8) daemon).
+/*	example, the pickup(8) or qmgr(8) daemon).
 /* .PP
 /*	Available in Postfix version 2.2 and later:
 /* .IP "\fBauthorized_flush_users (static:anyone)\fR"
@@ -189,7 +187,6 @@
 #include <flush_clnt.h>
 #include <smtp_stream.h>
 #include <user_acl.h>
-#include <valid_mailhost_addr.h>
 
 /* Application-specific. */
 
@@ -222,8 +219,8 @@
  /*
   * Queue manipulation access lists.
   */
-char   *var_flush_acl;
-char   *var_showq_acl;
+static char *var_flush_acl;
+static char *var_showq_acl;
 
 static CONFIG_STR_TABLE str_table[] = {
     VAR_FLUSH_ACL, DEF_FLUSH_ACL, &var_flush_acl, 0, 0,
@@ -451,7 +448,6 @@ int     main(int argc, char **argv)
      * Further initialization...
      */
     mail_conf_read();
-    mail_dict_init();				/* proxy, sql, ldap */
     get_mail_conf_str_table(str_table);
 
     /*
@@ -478,10 +474,14 @@ int     main(int argc, char **argv)
      */
     if (site_to_flush != 0) {
 	bad_site = 0;
-	if (*site_to_flush == '[') {
-	    bad_site = !valid_mailhost_literal(site_to_flush, DONT_GRIPE);
+	if (*site_to_flush == '['
+	    && *(last = site_to_flush + strlen(site_to_flush) - 1) == ']') {
+	    *last = 0;
+	    bad_site = !valid_hostaddr(site_to_flush + 1, DONT_GRIPE);
+	    *last = ']';
 	} else {
-	    bad_site = !valid_hostname(site_to_flush, DONT_GRIPE);
+	    bad_site = (!valid_hostname(site_to_flush, DONT_GRIPE)
+			&& !valid_hostaddr(site_to_flush, DONT_GRIPE));
 	}
 	if (bad_site)
 	    msg_fatal_status(EX_USAGE,
