@@ -35,7 +35,6 @@
 /*
 /*	smtp_reuse_addr() looks up a cached session by its server
 /*	address, and verifies that the session is still alive.
-/*	Lookup is disabled when the tls_per_site policy table is enabled.
 /*	The result is null in case of failure.
 /*
 /*	Arguments:
@@ -177,22 +176,6 @@ static SMTP_SESSION *smtp_reuse_common(SMTP_STATE *state, int fd,
     }
     state->session = session;
 
-#ifdef USE_TLS
-
-    /*
-     * Cached sessions are never TLS encrypted, so they must not be reused
-     * when TLS encryption is required.
-     */
-    if (session->tls_enforce_tls) {
-	if (msg_verbose)
-	    msg_info("%s: skipping plain-text cached session to %s",
-		     myname, label);
-	smtp_quit(state);			/* Close politely */
-	smtp_session_free(session);		/* And avoid leaks */
-	return (state->session = 0);
-    }
-#endif
-
     /*
      * Send an RSET probe to verify that the session is still good.
      */
@@ -242,19 +225,10 @@ SMTP_SESSION *smtp_reuse_domain(SMTP_STATE *state, int lookup_mx,
 
 SMTP_SESSION *smtp_reuse_addr(SMTP_STATE *state, DNS_RR *addr, unsigned port)
 {
-    MAI_HOSTADDR_STR hostaddr;
     SMTP_SESSION *session;
     int     fd;
 
-    /*
-     * XXX Disable connection cache lookup by server IP address when the
-     * tls_per_site policy or smtp_sasl_password_maps features are enabled.
-     * Different server names may resolve to the same IP address. We don't
-     * want to use the wrong SASL credentials or the wrong TLS policy.
-     */
-    if ((var_smtp_tls_per_site && *var_smtp_tls_per_site)
-	|| (var_smtp_sasl_passwd && *var_smtp_sasl_passwd))
-	return (0);
+#define INADDRP(x) ((struct in_addr *) (x))
 
     /*
      * Look up the session by its IP address. This means that we have no
@@ -263,10 +237,11 @@ SMTP_SESSION *smtp_reuse_addr(SMTP_STATE *state, DNS_RR *addr, unsigned port)
      * Note: if the label needs to be made more specific (with e.g., SASL login
      * information), just append the text with vstring_sprintf_append().
      */
-    if (dns_rr_to_pa(addr, &hostaddr) == 0)
+    if (addr->data_len > sizeof(struct in_addr))
 	return (0);
     vstring_sprintf(state->endp_label, SMTP_SCACHE_LABEL(NO_MX_LOOKUP),
-		    state->service, hostaddr.buf, ntohs(port));
+		    state->service, inet_ntoa(*INADDRP(addr->data)),
+		    ntohs(port));
     if ((fd = scache_find_endp(smtp_scache, STR(state->endp_label),
 			       state->endp_prop)) < 0)
 	return (0);
